
#include <iostream>

#include "abstractDataTypes/SubsetUnion.hpp"
#include "CityModel/CityModelFactory.hpp"
#include "CityModel/CityModelPackage.hpp"

#include "CityModel/Building.hpp"
#include "CityModel/City.hpp"
#include "CityModel/Person.hpp"
#include "CityModel/PublicBuilding.hpp"
#include "CityModel/ResidentialBuilding.hpp"

//Includes from main behavior mainProgram
#include "CityModel/Building.hpp"
#include "CityModel/City.hpp"
#include "CityModel/Person.hpp"
#include "CityModel/PublicBuilding.hpp"
#include "CityModel/ResidentialBuilding.hpp"


// Start of user code includes 
// You may manually edit additional includes, won't be overwritten upon generation.

// End of user code

using namespace CityModel;

// Start of user code functions 
// You may manually edit additional functions, won't be overwritten upon generation.

// End of user code

int main ()
{
	//Create Model Factory and Package
	std::shared_ptr<CityModelFactory> factory = CityModelFactory::eInstance();
	std::shared_ptr<CityModelPackage> package = CityModelPackage::eInstance();

	//Creating a new instance of City via model factory
std::shared_ptr<City> city =factory->createCity();

//Setting city name via console input
std::cout << "Enter a city name: ";
std::string cityName;
std::getline(std::cin, cityName);
city->setCityName(cityName);

//Every command that is entered is stored in the input_str
//Initially, the string is empty
std::string input_str ="";

//main loop of the program, can be exited by entering "stop"
while(true) {
  //after "Enter command: " is printed, one of the following commands can be entered
  std::cout << "Enter command: ";
  std::getline(std::cin, input_str);
  
  //ALL FOLLOWING CODE SHOULD BE PLACED IN HERE
//when typing "help", every available command will be printed to the console
//along with a short description
if (input_str=="help") {
  std::cout << "addPerson\t\t adds a person to the city" << std::endl;
  std::cout << "addPublicBuilding\t creates a public building with an address and a purpose in the city" << std::endl;
  std::cout << "addResidentialBuilding\t creates a residential building with an address in the city" << std::endl;
  std::cout << "changeWorkplace\t\t changes the workplace of a person" << std::endl;
  std::cout << "changeResidence\t\t changes the place of residence of a person" << std::endl;
  std::cout << "print\t\t\t prints all existing objects" << std::endl;
  std::cout << "stop\t\t\t exits the program" << std::endl;
}

//when typing "addPerson" and pressing enter, you can add a person to the city
//For that, you also have to type a name in the next line
else if (input_str=="addPerson") {
  //Creating a person in city via model factory
  std::shared_ptr<Person> person1 = factory->createPerson_as_persons_in_City(city);
  std::cout << "Name of person: ";
  //Setting name of the newly created person
  std::string pName;
  std::getline(std::cin, pName);
  person1->setName(pName);
}

//when typing "addPublicBuilding" and pressing enter, you can add a public building
//to the city
//For that, you have to type an address in the next line
//Also, you have to type a purpose in the next line (e.g. "Mall" or "Hospital")
else if (input_str=="addPublicBuilding") {
  //Creating a public building in city via model factory
  std::shared_ptr<PublicBuilding> pBuilding = factory -> 
      createPublicBuilding_as_publicBuildings_in_City(city);
  std::cout << "Address of public building: ";
  //Setting address of the newly created public building
  std::string address;
  std::getline(std::cin, address);
  pBuilding->setAddress(address);
  std::cout << "Purpose of public building: ";
  //Setting purpose of the newly created public building
  std::string purpose;
  std::getline(std::cin, purpose);
  pBuilding->setPurpose(purpose);
}

//when typing "addResidentialBuilding" and pressing enter, you can add
//a residential building to the city
//For that, you have to type an address in the next line
else if (input_str=="addResidentialBuilding") {
  //Creating a residential building in city via model factory
  std::shared_ptr<ResidentialBuilding> rBuilding = factory -> 
      createResidentialBuilding_as_residentialBuildings_in_City(city);
  std::cout << "Address of residential building: ";
  //Setting address of the newly created residential building
  std::string address;
  std::getline(std::cin, address);
  rBuilding->setAddress(address);
}

//when typing "changeResidence" and pressing enter, you can add a person as resident
//to a residential building (and removing him from the previous residential building)
//For that, you have to type the name of the person in question in the next line 
//Also, you have to type an address of a residential building in the next line
else if (input_str=="changeResidence") {
  //First input: name of person
  std::cout << "Name of resident: ";
  std::string name;
  std::getline(std::cin, name);

  //"resident = nullptr": placeholder for the pointer of the (already created) person,
  //which is to find
  std::shared_ptr<Person> resident = nullptr;
  //Get all persons from city and search for person with the same name as the input string
  std::shared_ptr<Bag<Person>> persons = city->getPersons();
  for(Bag<Person>::const_iterator it=persons->begin();it!=persons->end();it++)
  {
    std::string itName = (*it)->getName();
    //check for every person if the name is equal to the input string
    //if true: nullptr is replaced with the pointer of the found person
    if (itName == name) {
      resident = (*it);
    }
  }

  //Second input: address of building, where the person will be added
  std::cout << "Address of residential building that will be the new residence: ";
  std::string address;
  std::getline(std::cin, address);

  //true if resident has been found in the list of persons of the city
  //if true, person is added to the list of residents of the building at the address
  //given in the second input
  //if false (i.e. no person with the given name exists), a short message is printed,
  //but nothing changes besides that
  if (resident != nullptr) {
    city->changeResidence(resident, address);
  } else {
    std::cout << "Person \"" << name << "\" not found. To assign a person as resident" <<
        "to a residential building, the person must have been created before!"<< std::endl;
  }
}

//when typing "changeWorkplace" and pressing enter, you can add a person as employee
//to a public building (and removing him from the previous public building)
//For that, you have to type the name of the person in question in the next line 
//Also, you have to type an address of a public building in the next line
else if (input_str=="changeWorkplace") {
  //First input: name of person
  std::cout << "Name of employee: ";
  std::string name;
  std::getline(std::cin, name);

  //"employee = nullptr": placeholder for the pointer of the (already created) person,
  //which is to find
  std::shared_ptr<Person> employee =nullptr;
  //Get all persons from city and search for person with the same name as the input string
  std::shared_ptr<Bag<Person>> persons = city->getPersons();
  for(Bag<Person>::const_iterator it=persons->begin();it!=persons->end();it++)
  {
    std::string itName = (*it)->getName();
    //check for every person if the name is equal to the input string
    //if true: nullptr is replaced with the pointer of the found person
    if (itName == name) {
      employee = (*it);
    }
  }

  //Second input: address of building, where the person will be added
  std::cout << "Address of public building that will be the new workplace: ";
  std::string address;
  std::getline(std::cin, address);

  //true if employee has been found in the list of persons of the city
  //if true, person is added to the list of employees of the building at the address
  //given in the second input
  //if false (i.e. no person with the given name exists), a short message is printed,
  //but nothing changes besides that
  if (employee != nullptr) {
    city->changeWorkplace(employee, address);
  } else {
    std::cout << "Person \"" << name << "\" not found. To assign a person as employee to"<<
        "a public building, the person must have been created beforehand!" << std::endl;

  }
}

//when typing "print" and pressing enter, all existing objects will be printed
//for more information, read the comments at CityImpl::printCityContents()
else if (input_str=="print") {
  city->printCityContents();
}

//when typing "stop" and pressing enter, the program's main loop will be exited
//and the program terminates
else if (input_str=="stop") {
  break;
}

//when typing anything else, a short message appears, indicating help by typing "help"
else {
  std::cout << "Unknown command! Type \"help\" for a list of supported commands."<<
      std::endl;
}

}
std::cout << "Program ends..." << std::endl;

	// Start of user code main
	// You may manually edit the following lines, won't be overwritten upon generation.

	// End of user code

    return 0;
}
