#include "CityModel/impl/BuildingImpl.hpp"

#ifdef NDEBUG
  #define DEBUG_MESSAGE(a) /**/
#else
  #define DEBUG_MESSAGE(a) a
#endif

#include <iostream>


#include "abstractDataTypes/Any.hpp"
#include "abstractDataTypes/SubsetUnion.hpp"
#include "util/util.hpp"
#include "uml/Property.hpp"
#include "uml/Operation.hpp"
#include "uml/Parameter.hpp"
#include "CityModel/CityModelFactory.hpp"
#include "CityModel/impl/CityModelPackageImpl.hpp"
#include "uml/Class.hpp"

//Types included from attributes, operation parameters, imports and composite owner classes

//Packges and Factories included from types of attributes, operation parameters, imports and composite owner classes

//Packages of included Enumerations

//Includes of PluginFramework (if required)

//Includes of OpaqueBevaiors (if required)

//Includes from InstanceValues (if required)

//Includes from Ports typed by interfaces (if required)

//Includes from roles of ConnectorEnds (if required)

using namespace CityModel;

//*********************************
// Constructor / Destructor
//*********************************
BuildingImpl::BuildingImpl()
{
	/*
	NOTE: Due to virtual inheritance, base class constrcutors may not be called correctly
	*/
	DEBUG_MESSAGE(std::cout<<"Building is created..."<<std::endl;)
	//***********************************
	// init Get Set
	//getter init
		//Property address
		m_getterMap.insert(std::pair<long long,std::function<Any()>>(1071565701,[this](){ return eAny(this->getAddress());}));
	
	
	//setter init
	//Property address
		m_setterMap.insert(std::pair<long long,std::function<void(Any)>>(1071565701,[this](Any object){this->setAddress(object->get<std::string>());}));
	
	
	
	
}


BuildingImpl::~BuildingImpl()
{
	DEBUG_MESSAGE(std::cout<<"Building is destroyed..."<<std::endl;)
}

BuildingImpl::BuildingImpl(const BuildingImpl & obj):BuildingImpl()
{
	*this = obj;
}

std::shared_ptr<ecore::EObject>  BuildingImpl::copy() const
{
	std::shared_ptr<BuildingImpl> element(new BuildingImpl());
	*element=(*this);
	element->setThisBuildingPtr(element);
	return element;
}

BuildingImpl& BuildingImpl::operator=(const BuildingImpl & obj)
{
	//call overloaded =Operator for each base class
	uml::ElementImpl::operator=(obj);
	//create copy of all Attributes
	#ifdef SHOW_COPIES
	std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\r\ncopy Building "<< this << "\r\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ " << std::endl;
	#endif
	instantiate();

	//copy attributes with no containment (soft copy)
	m_address = obj.getAddress();

	//clone attributes with containment (deep copy)

	return *this;
}


std::shared_ptr<uml::Class> BuildingImpl::getMetaClass()
{
	return CityModelPackageImpl::eInstance()->get_CityModel_Building();
}

void BuildingImpl::instantiate()
{   
	
}

void BuildingImpl::destroy()
{	

	//Erase properties
	
	//Erase back reference to owner
}

//*********************************
// Attribute Setter Getter
//*********************************
void BuildingImpl::setAddress(std::string _address)
{
	m_address = _address;
	
}
std::string BuildingImpl::getAddress() const 
{

	return m_address;
}

//*********************************
// Union Getter
//*********************************



//*********************************
// Operations
//*********************************

//*********************************
// Structural Feature Getter/Setter
//*********************************
//Get
Any BuildingImpl::get(std::shared_ptr<uml::Property> _property) const
{
	std::string qualifiedName = _property->getQualifiedName();
    return this->get(qualifiedName);
}

Any BuildingImpl::get(std::string _qualifiedName) const
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    return this->get(uID);
}

Any BuildingImpl::get(long long _uID) const
{
	std::map<long long, std::function<Any()>>::const_iterator iter = m_getterMap.find(_uID);
    if(iter != m_getterMap.cend())
    {
        //invoke the getter function
        return iter->second();
    }

	return eAny(nullptr);
}

//Set
void BuildingImpl::set(std::shared_ptr<uml::Property> _property, Any value)
{
	std::string qualifiedName = _property->getQualifiedName();
    this->set(qualifiedName, value);
}

void BuildingImpl::set(std::string _qualifiedName, Any value)
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    this->set(uID, value);
}

void BuildingImpl::set(long long _uID, Any value)
{
	std::map<long long, std::function<void(Any)>>::const_iterator iter = m_setterMap.find(_uID);
    if(iter != m_setterMap.cend())
    {
        //invoke the setter function
        iter->second(value);
    }
}

//Unset
void BuildingImpl::unset(std::shared_ptr<uml::Property> _property)
{
	std::string qualifiedName = _property->getQualifiedName();
    this->unset(qualifiedName);
}

void BuildingImpl::unset(std::string _qualifiedName)
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    this->unset(uID);
}

void BuildingImpl::unset(long long _uID)
{
	std::map<long long, std::function<void()>>::const_iterator iter = m_unsetterMap.find(_uID);
    if(iter != m_unsetterMap.cend())
    {
        //invoke the unsetter function
        iter->second();
    }
}


//*********************************
// Operation Invoction
//*********************************
//Invoke
Any BuildingImpl::invoke(std::shared_ptr<uml::Operation> _operation, std::shared_ptr<Bag<Any>> _arguments)
{
	std::string qualifiedName = _operation->getQualifiedName();

	for(unsigned int i = 0; i < _operation->getOwnedParameter()->size(); i++)
	{
		qualifiedName += "_" + _operation->getOwnedParameter()->at(i)->getType()->getName();
	}

    return this->invoke(qualifiedName, _arguments);
}

Any BuildingImpl::invoke(std::string _qualifiedName, std::shared_ptr<Bag<Any>> _arguments)
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    return this->invoke(uID, _arguments);
}

Any BuildingImpl::invoke(long long _uID, std::shared_ptr<Bag<Any>> _arguments)
{
	std::map<long long, std::function<Any(std::shared_ptr<Bag<Any>>)>>::const_iterator iter = m_invocationMap.find(_uID);
    if(iter != m_invocationMap.cend())
    {
        //invoke the operation
        return iter->second(_arguments);
    }
	
	return eAny(nullptr);
}

std::shared_ptr<Building> BuildingImpl::getThisBuildingPtr()
{
	return m_thisBuildingPtr.lock();
}
void BuildingImpl::setThisBuildingPtr(std::weak_ptr<Building> thisBuildingPtr)
{
	m_thisBuildingPtr = thisBuildingPtr;
	setThisElementPtr(thisBuildingPtr);
}
