#include "CityModel/impl/CityImpl.hpp"

#ifdef NDEBUG
  #define DEBUG_MESSAGE(a) /**/
#else
  #define DEBUG_MESSAGE(a) a
#endif

#include <iostream>

#include "abstractDataTypes/Bag.hpp"

#include "abstractDataTypes/Any.hpp"
#include "abstractDataTypes/SubsetUnion.hpp"
#include "util/util.hpp"
#include "uml/Property.hpp"
#include "uml/Operation.hpp"
#include "uml/Parameter.hpp"
#include "CityModel/CityModelFactory.hpp"
#include "CityModel/impl/CityModelPackageImpl.hpp"
#include "uml/Class.hpp"

//Types included from attributes, operation parameters, imports and composite owner classes
#include "CityModel/Person.hpp"
#include "CityModel/PublicBuilding.hpp"
#include "CityModel/ResidentialBuilding.hpp"

//Packges and Factories included from types of attributes, operation parameters, imports and composite owner classes

//Packages of included Enumerations

//Includes of PluginFramework (if required)

//Includes of OpaqueBevaiors (if required)
//Included from operation "changeResidence"
#include "CityModel\ResidentialBuilding.hpp"
#include "CityModel\Person.hpp"

//Included from operation "changeWorkplace"
#include "CityModel\PublicBuilding.hpp"
#include "CityModel\Person.hpp"

//Included from operation "printCityContents"
#include "CityModel\Building.hpp"
#include "CityModel\Person.hpp"


//Includes from InstanceValues (if required)

//Includes from Ports typed by interfaces (if required)

//Includes from roles of ConnectorEnds (if required)

using namespace CityModel;

//*********************************
// Constructor / Destructor
//*********************************
CityImpl::CityImpl()
{
	/*
	NOTE: Due to virtual inheritance, base class constrcutors may not be called correctly
	*/
	DEBUG_MESSAGE(std::cout<<"City is created..."<<std::endl;)
	//***********************************
	// init Get Set
	//getter init
		//Property cityName
		m_getterMap.insert(std::pair<long long,std::function<Any()>>(2131604989,[this](){ return eAny(this->getCityName());}));
		//Property persons
		m_getterMap.insert(std::pair<long long,std::function<Any()>>(170171026,[this](){ return eAny(this->getPersons());}));
		//Property publicBuildings
		m_getterMap.insert(std::pair<long long,std::function<Any()>>(1971921328,[this](){ return eAny(this->getPublicBuildings());}));
		//Property residentialBuildings
		m_getterMap.insert(std::pair<long long,std::function<Any()>>(1949948126,[this](){ return eAny(this->getResidentialBuildings());}));
	
	
	//setter init
	//Property cityName
		m_setterMap.insert(std::pair<long long,std::function<void(Any)>>(2131604989,[this](Any object){this->setCityName(object->get<std::string>());}));
	
	
	//unsetter init
		//Property persons
		m_unsetterMap.insert(std::pair<long long,std::function<void()>>(170171026,[this](){m_persons->clear();}));
		//Property publicBuildings
		m_unsetterMap.insert(std::pair<long long,std::function<void()>>(1971921328,[this](){m_publicBuildings->clear();}));
		//Property residentialBuildings
		m_unsetterMap.insert(std::pair<long long,std::function<void()>>(1949948126,[this](){m_residentialBuildings->clear();}));
	
	
	//invocation init
		//Operation printCityContents
		m_invocationMap.insert(std::pair<long long, std::function<Any(std::shared_ptr<Bag<Any>>)>>(637190611, [this](std::shared_ptr<Bag<Any>> arguments) {
			this->printCityContents();
			return eAny(nullptr);
		}));
		//Operation changeResidence
		m_invocationMap.insert(std::pair<long long, std::function<Any(std::shared_ptr<Bag<Any>>)>>(1794571174, [this](std::shared_ptr<Bag<Any>> arguments) {
			//Retrieve input parameter 'person'
			Any anyPerson = *(arguments->at(0).get());
			std::shared_ptr<CityModel::Person> person = anyPerson->get<std::shared_ptr<CityModel::Person>>();
			//Retrieve input parameter 'residentialBuildingAddress'
			Any anyResidentialBuildingAddress = *(arguments->at(1).get());
			std::string residentialBuildingAddress = anyResidentialBuildingAddress->get<std::string>();
	
			this->changeResidence(person, residentialBuildingAddress);
			return eAny(nullptr);
		}));
		//Operation changeWorkplace
		m_invocationMap.insert(std::pair<long long, std::function<Any(std::shared_ptr<Bag<Any>>)>>(33697662, [this](std::shared_ptr<Bag<Any>> arguments) {
			//Retrieve input parameter 'person'
			Any anyPerson = *(arguments->at(0).get());
			std::shared_ptr<CityModel::Person> person = anyPerson->get<std::shared_ptr<CityModel::Person>>();
			//Retrieve input parameter 'publicBuildingAddress'
			Any anyPublicBuildingAddress = *(arguments->at(1).get());
			std::string publicBuildingAddress = anyPublicBuildingAddress->get<std::string>();
	
			this->changeWorkplace(person, publicBuildingAddress);
			return eAny(nullptr);
		}));
	
}


CityImpl::~CityImpl()
{
	DEBUG_MESSAGE(std::cout<<"City is destroyed..."<<std::endl;)
}

CityImpl::CityImpl(const CityImpl & obj):CityImpl()
{
	*this = obj;
}

std::shared_ptr<ecore::EObject>  CityImpl::copy() const
{
	std::shared_ptr<CityImpl> element(new CityImpl());
	*element=(*this);
	element->setThisCityPtr(element);
	return element;
}

CityImpl& CityImpl::operator=(const CityImpl & obj)
{
	//call overloaded =Operator for each base class
	uml::ElementImpl::operator=(obj);
	//create copy of all Attributes
	#ifdef SHOW_COPIES
	std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\r\ncopy City "<< this << "\r\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ " << std::endl;
	#endif
	instantiate();

	//copy attributes with no containment (soft copy)
	m_cityName = obj.getCityName();

	//clone attributes with containment (deep copy)
	std::shared_ptr<Bag<CityModel::Person>> personsList = obj.getPersons();
	Bag<CityModel::Person>::iterator personsIter = personsList->begin();
	Bag<CityModel::Person>::iterator personsEnd = personsList->end();
	while (personsIter != personsEnd) 
	{
		std::shared_ptr<CityModel::Person> temp = std::dynamic_pointer_cast<CityModel::Person>((*personsIter)->copy());
		getPersons()->push_back(temp);
		personsIter++;
	}
	std::shared_ptr<Bag<CityModel::PublicBuilding>> publicBuildingsList = obj.getPublicBuildings();
	Bag<CityModel::PublicBuilding>::iterator publicBuildingsIter = publicBuildingsList->begin();
	Bag<CityModel::PublicBuilding>::iterator publicBuildingsEnd = publicBuildingsList->end();
	while (publicBuildingsIter != publicBuildingsEnd) 
	{
		std::shared_ptr<CityModel::PublicBuilding> temp = std::dynamic_pointer_cast<CityModel::PublicBuilding>((*publicBuildingsIter)->copy());
		getPublicBuildings()->push_back(temp);
		publicBuildingsIter++;
	}
	std::shared_ptr<Bag<CityModel::ResidentialBuilding>> residentialBuildingsList = obj.getResidentialBuildings();
	Bag<CityModel::ResidentialBuilding>::iterator residentialBuildingsIter = residentialBuildingsList->begin();
	Bag<CityModel::ResidentialBuilding>::iterator residentialBuildingsEnd = residentialBuildingsList->end();
	while (residentialBuildingsIter != residentialBuildingsEnd) 
	{
		std::shared_ptr<CityModel::ResidentialBuilding> temp = std::dynamic_pointer_cast<CityModel::ResidentialBuilding>((*residentialBuildingsIter)->copy());
		getResidentialBuildings()->push_back(temp);
		residentialBuildingsIter++;
	}

	return *this;
}


std::shared_ptr<uml::Class> CityImpl::getMetaClass()
{
	return CityModelPackageImpl::eInstance()->get_CityModel_City();
}

void CityImpl::instantiate()
{   
	
	//instantiate property 'persons'
	m_persons.reset(new Bag<CityModel::Person>());
	
	
	//instantiate property 'publicBuildings'
	m_publicBuildings.reset(new Bag<CityModel::PublicBuilding>());
	
	
	//instantiate property 'residentialBuildings'
	m_residentialBuildings.reset(new Bag<CityModel::ResidentialBuilding>());
	
	
}

void CityImpl::destroy()
{	

	//Erase properties
	
	//deleting property persons
	//property is part or port and is destroyed recursively
	for(unsigned int i = 0; i < m_persons->size(); i++)
	{ 
		if(m_persons->at(i)) m_persons->at(i)->destroy();  
	}
	m_persons->clear();
	
	//deleting property publicBuildings
	//property is part or port and is destroyed recursively
	for(unsigned int i = 0; i < m_publicBuildings->size(); i++)
	{ 
		if(m_publicBuildings->at(i)) m_publicBuildings->at(i)->destroy();  
	}
	m_publicBuildings->clear();
	
	//deleting property residentialBuildings
	//property is part or port and is destroyed recursively
	for(unsigned int i = 0; i < m_residentialBuildings->size(); i++)
	{ 
		if(m_residentialBuildings->at(i)) m_residentialBuildings->at(i)->destroy();  
	}
	m_residentialBuildings->clear();
	
	//Erase back reference to owner
}

//*********************************
// Attribute Setter Getter
//*********************************
void CityImpl::setCityName(std::string _cityName)
{
	m_cityName = _cityName;
	
}
std::string CityImpl::getCityName() const 
{

	return m_cityName;
}


std::shared_ptr<Bag<CityModel::Person>> CityImpl::getPersons() const 
{
	if(m_persons == nullptr)
	{
		m_persons.reset(new Bag<CityModel::Person>());
		
	}

	return m_persons;
}


std::shared_ptr<Bag<CityModel::PublicBuilding>> CityImpl::getPublicBuildings() const 
{
	if(m_publicBuildings == nullptr)
	{
		m_publicBuildings.reset(new Bag<CityModel::PublicBuilding>());
		
	}

	return m_publicBuildings;
}


std::shared_ptr<Bag<CityModel::ResidentialBuilding>> CityImpl::getResidentialBuildings() const 
{
	if(m_residentialBuildings == nullptr)
	{
		m_residentialBuildings.reset(new Bag<CityModel::ResidentialBuilding>());
		
	}

	return m_residentialBuildings;
}

//*********************************
// Union Getter
//*********************************



//*********************************
// Operations
//*********************************
void CityImpl::changeResidence(std::shared_ptr<CityModel::Person> person,std::string residentialBuildingAddress) 
{

	// Implemented as Opaque behaviour changeResidence

//this method changes the home of a person

//all residential buildings and persons of the city
std::shared_ptr<Bag<ResidentialBuilding>> rBuildings = this -> getResidentialBuildings();
std::shared_ptr<Bag<Person>> persons = this -> getPersons();

//first, we need to find the correct residential building for the given address
//therefore, we iterate through all residential buildings of the city
for(Bag<ResidentialBuilding>::const_iterator it=rBuildings -> begin();
    it!=rBuildings -> end(); it++)
{
  //removing a person from a residential building (his previous home)
  //check whether there is a resident with the same name as the input
  //(by iterating through all residents and comparing their names)
  std::shared_ptr< Bag < Person > > residents = (*it) -> getInhabitants();
  for(Bag<Person>::const_iterator pers_it=residents -> begin();
      pers_it!=residents -> end(); pers_it++)
  {
    //if there is a person with the same name, he is removed from the residential
    //building (his previous home)
    if ((*pers_it) -> getName() == person -> getName()) {
      //removing the person from the list of residents of the building
      (*it) -> removeResident(person);
      std::cout << "Removed resident " << person -> getName() << " from " <<
          (*it) -> getAddress() <<  std::endl;
      break;
    }
  }
  //adding a person as resident to a building (his new home)
  //only if a residential building with the same address as the given address exists
  if ((*it) -> getAddress() == residentialBuildingAddress) {
    //adds the person to the list of residents of the building
    (*it) -> addResident(person);
    std::cout << "Added resident " << person -> getName() << " to " <<
        (*it) -> getAddress() <<  std::endl;
  }
}


}

void CityImpl::changeWorkplace(std::shared_ptr<CityModel::Person> person,std::string publicBuildingAddress) 
{

	// Implemented as Opaque behaviour changeWorkplace

//this method changes the workplace of a person

//get all public buildings and persons of the city
std::shared_ptr<Bag<PublicBuilding>> pBuildings = this -> getPublicBuildings();
std::shared_ptr<Bag<Person>> persons = this -> getPersons();

//first, we need to find the correct public building for the given address
//therefore, we iterate through all public buildings of the city
for(Bag<PublicBuilding>::const_iterator it=pBuildings->begin();it!=pBuildings->end();it++)
{
  //removing a person from a public building (his previous workplace)
  //check whether there is an employee with the same name as the input
  //(by iterating through all employees and comparing their names)
  std::shared_ptr<Bag<Person>> employees = (*it) -> getEmployees();
  for(Bag<Person>::const_iterator pers_it=employees -> begin();
      pers_it!=employees -> end(); pers_it++)
  {
    //if there is a person with the same name, he is removed from the public building 
    //(his previous workplace)
    if ((*pers_it) -> getName() == person -> getName()) {
      //removing the person from the list of employees of the building
      (*it) -> removeEmployee(person);
      std::cout << "Removed employee " << person -> getName() << " from " << (*it) ->
      getAddress() <<  std::endl;
      break;
    }
  }
    //adding a person as employee to a building (his new workplace)
    //only if a public building with the same address as the given address exists
    if ((*it) -> getAddress() == publicBuildingAddress) {
      //adds the person to the list of employees of the building
      (*it) -> addEmployee(person);
      std::cout << "Added employee " << person -> getName() << " to " << (*it) ->
      getAddress() <<  std::endl;
    }
  }



}

void CityImpl::printCityContents() 
{

	// Implemented as Opaque behaviour printCityContents

//every existing object of the city must be printed
//therefore, we need to collect all objects first
std::shared_ptr< Bag<PublicBuilding> > publicBuildings = this ->getPublicBuildings();
std::shared_ptr< Bag<ResidentialBuilding> > residentialBuildings = this ->
    getResidentialBuildings();
std::shared_ptr< Bag<Person> > persons = this -> getPersons();

std::cout << "========================================" << std::endl;
std::cout << "City " << this -> getCityName() << " contains: " << std::endl;
std::cout << "========================================" << std::endl;

std::cout << "Public Buildings:" << std::endl;
std::cout << "----------------------------------------" << std::endl;

//iterating through all public buildings, printing its address, purpose, and all employees
for(Bag<PublicBuilding>::const_iterator it=publicBuildings -> begin();
    it!=publicBuildings -> end(); it++)
{
  std::cout << "\tPublic Building with address \"" << (*it) -> getAddress() <<
      "\" and purpose \"" << (*it) -> getPurpose() << "\"" <<  std::endl;
  std::cout << "\t\tEmployees:" << std::endl;
  //list of employees in the current public building
  std::shared_ptr< Bag<Person> > persons = (*it) -> getEmployees();
  //iterating through the list of employees in the current public building
  //and printing their names
  for(Bag<Person>::const_iterator itPerson=persons -> begin();
      itPerson!=persons -> end(); itPerson++)
  {
    std::cout << "\t\t\tPerson " << (*itPerson) -> getName() << std::endl;
  }
}

std::cout << "Residential Buildings:" << std::endl;
std::cout << "----------------------------------------" << std::endl;

//iterating through all residential buildings, printing its address and all residents
for(Bag<ResidentialBuilding>::const_iterator it=residentialBuildings -> begin();
    it!=residentialBuildings -> end(); it++)
{
  std::cout << "\tResidential Building with address \"" << (*it) -> getAddress() <<
      "\"" <<  std::endl;
  std::cout << "\t\tResidents:" << std::endl;
  //list of residents in the current residential building
  std::shared_ptr< Bag<Person> > persons = (*it) -> getInhabitants();
  //iterating through the list of residents in the current residential building
  //and printing their names
  for(Bag<Person>::const_iterator itPerson=persons -> begin();
      itPerson!=persons -> end(); itPerson++)
  {
    std::cout << "\t\t\tPerson " << (*itPerson) -> getName() << std::endl;
  }
}

std::cout << "Persons in city:" << std::endl;
std::cout << "----------------------------------------" << std::endl;

//iterating through all persons of the city and printing their names
for(Bag<Person>::const_iterator it=persons -> begin(); it!=persons -> end(); it++)
{
  std::cout << "\tPerson with name: " << (*it) -> getName() << std::endl;
}
std::cout << "========================================"<< std::endl;
std::cout << "Printed everything from City " << this -> getCityName() << std::endl;
std::cout << "========================================"<< std::endl;


}

//*********************************
// Structural Feature Getter/Setter
//*********************************
//Get
Any CityImpl::get(std::shared_ptr<uml::Property> _property) const
{
	std::string qualifiedName = _property->getQualifiedName();
    return this->get(qualifiedName);
}

Any CityImpl::get(std::string _qualifiedName) const
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    return this->get(uID);
}

Any CityImpl::get(long long _uID) const
{
	std::map<long long, std::function<Any()>>::const_iterator iter = m_getterMap.find(_uID);
    if(iter != m_getterMap.cend())
    {
        //invoke the getter function
        return iter->second();
    }

	return eAny(nullptr);
}

//Set
void CityImpl::set(std::shared_ptr<uml::Property> _property, Any value)
{
	std::string qualifiedName = _property->getQualifiedName();
    this->set(qualifiedName, value);
}

void CityImpl::set(std::string _qualifiedName, Any value)
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    this->set(uID, value);
}

void CityImpl::set(long long _uID, Any value)
{
	std::map<long long, std::function<void(Any)>>::const_iterator iter = m_setterMap.find(_uID);
    if(iter != m_setterMap.cend())
    {
        //invoke the setter function
        iter->second(value);
    }
}

//Unset
void CityImpl::unset(std::shared_ptr<uml::Property> _property)
{
	std::string qualifiedName = _property->getQualifiedName();
    this->unset(qualifiedName);
}

void CityImpl::unset(std::string _qualifiedName)
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    this->unset(uID);
}

void CityImpl::unset(long long _uID)
{
	std::map<long long, std::function<void()>>::const_iterator iter = m_unsetterMap.find(_uID);
    if(iter != m_unsetterMap.cend())
    {
        //invoke the unsetter function
        iter->second();
    }
}


//*********************************
// Operation Invoction
//*********************************
//Invoke
Any CityImpl::invoke(std::shared_ptr<uml::Operation> _operation, std::shared_ptr<Bag<Any>> _arguments)
{
	std::string qualifiedName = _operation->getQualifiedName();

	for(unsigned int i = 0; i < _operation->getOwnedParameter()->size(); i++)
	{
		qualifiedName += "_" + _operation->getOwnedParameter()->at(i)->getType()->getName();
	}

    return this->invoke(qualifiedName, _arguments);
}

Any CityImpl::invoke(std::string _qualifiedName, std::shared_ptr<Bag<Any>> _arguments)
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    return this->invoke(uID, _arguments);
}

Any CityImpl::invoke(long long _uID, std::shared_ptr<Bag<Any>> _arguments)
{
	std::map<long long, std::function<Any(std::shared_ptr<Bag<Any>>)>>::const_iterator iter = m_invocationMap.find(_uID);
    if(iter != m_invocationMap.cend())
    {
        //invoke the operation
        return iter->second(_arguments);
    }
	
	return eAny(nullptr);
}

std::shared_ptr<City> CityImpl::getThisCityPtr()
{
	return m_thisCityPtr.lock();
}
void CityImpl::setThisCityPtr(std::weak_ptr<City> thisCityPtr)
{
	m_thisCityPtr = thisCityPtr;
	setThisElementPtr(thisCityPtr);
}
