#include "CityModel/impl/CityModelFactoryImpl.hpp"

#include "abstractDataTypes/SubsetUnion.hpp"
#include "uml/Class.hpp"
#include <cassert>

#include "CityModel/impl/BuildingImpl.hpp"
#include "CityModel/impl/CityImpl.hpp"
#include "CityModel/impl/PersonImpl.hpp"
#include "CityModel/impl/PublicBuildingImpl.hpp"
#include "CityModel/impl/ResidentialBuildingImpl.hpp"


using namespace CityModel;
//*********************************
// Constructor / Destructor
//*********************************
CityModelFactoryImpl::CityModelFactoryImpl()
{
	m_idMap.insert(std::make_pair("CityModel::City", CityModelPackage::CITY_CLASS));
	m_idMap.insert(std::make_pair("CityModel::Person", CityModelPackage::PERSON_CLASS));
	m_idMap.insert(std::make_pair("CityModel::PublicBuilding", CityModelPackage::PUBLICBUILDING_CLASS));
	m_idMap.insert(std::make_pair("CityModel::ResidentialBuilding", CityModelPackage::RESIDENTIALBUILDING_CLASS));
}

CityModelFactoryImpl::~CityModelFactoryImpl()
{
}

CityModelFactory* CityModelFactoryImpl::create()
{
	return new CityModelFactoryImpl();
}

//*********************************
// creators
//*********************************
std::shared_ptr<uml::Element> CityModelFactoryImpl::create(const unsigned int _metaClassId,  std::shared_ptr<uml::Element> container /*= nullptr*/, const unsigned int referenceID/* = -1*/) const
{
	switch(_metaClassId)
	{
		case CityModelPackage::CITY_CLASS:
		{
				return this->createCity(_metaClassId);
			
		}
		case CityModelPackage::PERSON_CLASS:
		{
			if (nullptr == container)
			{
				return this->createPerson(_metaClassId);
			}
			else
			{
				std::shared_ptr<CityModel::City> castedContainer = std::dynamic_pointer_cast<CityModel::City>(container);
				assert(castedContainer);
				return std::shared_ptr<Person>(this->createPerson_as_persons_in_City(castedContainer,_metaClassId));
			}
		}
		case CityModelPackage::PUBLICBUILDING_CLASS:
		{
			if (nullptr == container)
			{
				return this->createPublicBuilding(_metaClassId);
			}
			else
			{
				std::shared_ptr<CityModel::City> castedContainer = std::dynamic_pointer_cast<CityModel::City>(container);
				assert(castedContainer);
				return std::shared_ptr<PublicBuilding>(this->createPublicBuilding_as_publicBuildings_in_City(castedContainer,_metaClassId));
			}
		}
		case CityModelPackage::RESIDENTIALBUILDING_CLASS:
		{
			if (nullptr == container)
			{
				return this->createResidentialBuilding(_metaClassId);
			}
			else
			{
				std::shared_ptr<CityModel::City> castedContainer = std::dynamic_pointer_cast<CityModel::City>(container);
				assert(castedContainer);
				return std::shared_ptr<ResidentialBuilding>(this->createResidentialBuilding_as_residentialBuildings_in_City(castedContainer,_metaClassId));
			}
		}
	default:
	   	    std::cerr << __PRETTY_FUNCTION__ << " ID " << _metaClassId <<" not found" << std::endl;
	}
	return nullptr;
}

std::shared_ptr<uml::Element> CityModelFactoryImpl::create(std::shared_ptr<uml::Class> _class, std::shared_ptr<uml::Element> container /*= nullptr*/, const unsigned int referenceID/* = -1*/) const
{
	if(_class->getIsAbstract())
    {
    	return nullptr;
    }
	std::string typeName = _class->getQualifiedName();
	return create(typeName,container,referenceID);
}

std::shared_ptr<uml::Element> CityModelFactoryImpl::create(std::string _className, std::shared_ptr<uml::Element> container /*= nullptr*/, const unsigned int referenceID/* = -1*/) const
{
	auto iter = m_idMap.find(_className);
	if(iter != m_idMap.end())
    {
		//get the getMetaElementId()
        unsigned int id = iter->second;
		return create(id,container,referenceID);
    }
    return nullptr;
}

//--------------------------
//create-Functions for class 'City'
//--------------------------
std::shared_ptr<CityModel::City> CityModelFactoryImpl::createCity(const int metaElementID/*=-1*/) const
{
	std::shared_ptr<CityModel::CityImpl> element(new CityImpl());
	element->setMetaElementID(metaElementID);
	element->setThisCityPtr(element);
	element->instantiate();
	return element;
}


//--------------------------
//create-Functions for class 'Person'
//--------------------------
std::shared_ptr<CityModel::Person> CityModelFactoryImpl::createPerson(const int metaElementID/*=-1*/) const
{
	std::shared_ptr<CityModel::PersonImpl> element(new PersonImpl());
	element->setMetaElementID(metaElementID);
	element->setThisPersonPtr(element);
	element->instantiate();
	return element;
}

std::shared_ptr<CityModel::Person> CityModelFactoryImpl::createPerson_as_persons_in_City(std::shared_ptr<CityModel::City> par_citys, const int metaElementID/*=-1*/) const
{
	std::shared_ptr<CityModel::PersonImpl> element(new PersonImpl());
	element->setMetaElementID(metaElementID);
	if (nullptr != par_citys)
	{
		par_citys->getPersons()->push_back(element);
	}
	element->setThisPersonPtr(element);
	element->instantiate();
	return element;
}

//--------------------------
//create-Functions for class 'PublicBuilding'
//--------------------------
std::shared_ptr<CityModel::PublicBuilding> CityModelFactoryImpl::createPublicBuilding(const int metaElementID/*=-1*/) const
{
	std::shared_ptr<CityModel::PublicBuildingImpl> element(new PublicBuildingImpl());
	element->setMetaElementID(metaElementID);
	element->setThisPublicBuildingPtr(element);
	element->instantiate();
	return element;
}

std::shared_ptr<CityModel::PublicBuilding> CityModelFactoryImpl::createPublicBuilding_as_publicBuildings_in_City(std::shared_ptr<CityModel::City> par_citys, const int metaElementID/*=-1*/) const
{
	std::shared_ptr<CityModel::PublicBuildingImpl> element(new PublicBuildingImpl());
	element->setMetaElementID(metaElementID);
	if (nullptr != par_citys)
	{
		par_citys->getPublicBuildings()->push_back(element);
	}
	element->setThisPublicBuildingPtr(element);
	element->instantiate();
	return element;
}

//--------------------------
//create-Functions for class 'ResidentialBuilding'
//--------------------------
std::shared_ptr<CityModel::ResidentialBuilding> CityModelFactoryImpl::createResidentialBuilding(const int metaElementID/*=-1*/) const
{
	std::shared_ptr<CityModel::ResidentialBuildingImpl> element(new ResidentialBuildingImpl());
	element->setMetaElementID(metaElementID);
	element->setThisResidentialBuildingPtr(element);
	element->instantiate();
	return element;
}

std::shared_ptr<CityModel::ResidentialBuilding> CityModelFactoryImpl::createResidentialBuilding_as_residentialBuildings_in_City(std::shared_ptr<CityModel::City> par_citys, const int metaElementID/*=-1*/) const
{
	std::shared_ptr<CityModel::ResidentialBuildingImpl> element(new ResidentialBuildingImpl());
	element->setMetaElementID(metaElementID);
	if (nullptr != par_citys)
	{
		par_citys->getResidentialBuildings()->push_back(element);
	}
	element->setThisResidentialBuildingPtr(element);
	element->instantiate();
	return element;
}




std::shared_ptr<CityModelPackage> CityModelFactoryImpl::getCityModelPackage()
{
	return CityModelPackage::eInstance();
}
