#include "CityModel/impl/CityModelPackageImpl.hpp"

//meta meta model classes
#include "uml/Association.hpp"
#include "uml/Class.hpp"
#include "uml/FunctionBehavior.hpp"
#include "uml/Generalization.hpp"
#include "uml/LiteralInteger.hpp"
#include "uml/LiteralUnlimitedNatural.hpp"
#include "uml/Operation.hpp"
#include "uml/PackageImport.hpp"
#include "uml/Parameter.hpp"
#include "uml/PrimitiveType.hpp"
#include "uml/ProfileApplication.hpp"
#include "uml/Property.hpp"

#include "abstractDataTypes/SubsetUnion.hpp"
#include "uml/VisibilityKind.hpp"
//meta meta model factory
#include "uml/umlFactory.hpp"

//metamodel factory
#include "CityModel/CityModelFactory.hpp"

//depending model packages
#include "primitivetypesReflection/PrimitiveTypesPackage.hpp"
#include "UML4CPPProfile/UML4CPPProfilePackage.hpp"
#include "UML4CPPProfile/UML4CPPProfileFactory.hpp"



//Stereotypes
#include "UML4CPPProfile/MainBehavior.hpp"

using namespace CityModel;

CityModelPackageImpl::CityModelPackageImpl()
{
}

CityModelPackageImpl::~CityModelPackageImpl()
{
}

bool CityModelPackageImpl::isInited = false;

CityModelPackage* CityModelPackageImpl::create()
{
	if (isInited)
	{
		return CityModelPackage::eInstance().get();
	}
	isInited = true;
	
    // Obtain or create and register package, create package meta-data objects
    CityModelPackageImpl * metaModelPackage = new CityModelPackageImpl();
    return metaModelPackage;
}

void CityModelPackageImpl::init(std::shared_ptr<uml::Package> cityModel)
{
    // Initialize created meta-data
	setThisPackagePtr(cityModel);
	initMetaModel();
    createPackageContents(cityModel);
    initializePackageContents(cityModel);   
}

void CityModelPackageImpl::initMetaModel()
{
}

void CityModelPackageImpl::createPackageContents(std::shared_ptr<uml::Package> cityModel)
{
	if (isCreated)
	{
		return;
	}
	isCreated = true;

	std::shared_ptr<uml::umlFactory> factory = uml::umlFactory::eInstance();

	createPackageValueSpecifications(cityModel, factory);
	createPackageInterfaces(cityModel, factory);
	createPackageClasses(cityModel, factory);
	createPackageAssociations(cityModel, factory);
	createPackageInstanceSpecifications(cityModel, factory);
	createPackageStereotypes(cityModel, factory);
	createPackageActivities(cityModel, factory);
	createPackageDependencies(cityModel, factory);
	createPackagePrimitiveTypes(cityModel, factory);
	createPackageEnumerationLiterals(cityModel, factory);
	createPackageInterfaceRealizations(cityModel, factory);
}

void CityModelPackageImpl::createPackageActivities(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
}


void CityModelPackageImpl::createPackageClasses(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
	std::shared_ptr<uml::Constraint> con = nullptr;
	std::shared_ptr<uml::OpaqueExpression> oe = nullptr;

	//Building
	cityModel_Building = factory->createClass_as_ownedType_in_Package(cityModel, CityModel::CityModelPackage::BUILDING_CLASS);
	
    // Building attributes
	//address
	cityModel_Building_address = factory->createProperty_as_ownedAttribute_in_Class(cityModel_Building);

    //-------------------------------------------
	//City
	cityModel_City = factory->createClass_as_ownedType_in_Package(cityModel, CityModel::CityModelPackage::CITY_CLASS);
	
    // City attributes
	//cityName
	cityModel_City_cityName = factory->createProperty_as_ownedAttribute_in_Class(cityModel_City);
	//persons
	cityModel_City_persons = factory->createProperty_as_ownedAttribute_in_Class(cityModel_City);
	//publicBuildings
	cityModel_City_publicBuildings = factory->createProperty_as_ownedAttribute_in_Class(cityModel_City);
	//residentialBuildings
	cityModel_City_residentialBuildings = factory->createProperty_as_ownedAttribute_in_Class(cityModel_City);
    // City operations
	//changeResidence
	cityModel_City_changeResidence_Person_String_Person_String = factory->createOperation_as_ownedOperation_in_Class(cityModel_City);
	cityModel_City_changeResidence_Person_String_Person_String_person = factory->createParameter_as_ownedParameter_in_Operation(cityModel_City_changeResidence_Person_String_Person_String);cityModel_City_changeResidence_Person_String_Person_String_residentialBuildingAddress = factory->createParameter_as_ownedParameter_in_Operation(cityModel_City_changeResidence_Person_String_Person_String);

	//changeWorkplace
	cityModel_City_changeWorkplace_Person_String_Person_String = factory->createOperation_as_ownedOperation_in_Class(cityModel_City);
	cityModel_City_changeWorkplace_Person_String_Person_String_person = factory->createParameter_as_ownedParameter_in_Operation(cityModel_City_changeWorkplace_Person_String_Person_String);cityModel_City_changeWorkplace_Person_String_Person_String_publicBuildingAddress = factory->createParameter_as_ownedParameter_in_Operation(cityModel_City_changeWorkplace_Person_String_Person_String);

	//printCityContents
	cityModel_City_printCityContents__ = factory->createOperation_as_ownedOperation_in_Class(cityModel_City);
    // City opaque behaviors
	//changeResidence
	cityModel_City_changeResidence = factory->createFunctionBehavior_as_ownedBehavior_in_BehavioredClassifier(cityModel_City);
	

	//changeWorkplace
	cityModel_City_changeWorkplace = factory->createFunctionBehavior_as_ownedBehavior_in_BehavioredClassifier(cityModel_City);
	

	//printCityContents
	cityModel_City_printCityContents = factory->createFunctionBehavior_as_ownedBehavior_in_BehavioredClassifier(cityModel_City);
	

    //-------------------------------------------
	//Person
	cityModel_Person = factory->createClass_as_ownedType_in_Package(cityModel, CityModel::CityModelPackage::PERSON_CLASS);
	
    // Person attributes
	//name
	cityModel_Person_name = factory->createProperty_as_ownedAttribute_in_Class(cityModel_Person);

    //-------------------------------------------
	//PublicBuilding
	cityModel_PublicBuilding = factory->createClass_as_ownedType_in_Package(cityModel, CityModel::CityModelPackage::PUBLICBUILDING_CLASS);
	
    // PublicBuilding attributes
	//employees
	cityModel_PublicBuilding_employees = factory->createProperty_as_ownedAttribute_in_Class(cityModel_PublicBuilding);
	//purpose
	cityModel_PublicBuilding_purpose = factory->createProperty_as_ownedAttribute_in_Class(cityModel_PublicBuilding);
    // PublicBuilding operations
	//addEmployee
	cityModel_PublicBuilding_addEmployee_Person_Person = factory->createOperation_as_ownedOperation_in_Class(cityModel_PublicBuilding);
	cityModel_PublicBuilding_addEmployee_Person_Person_person = factory->createParameter_as_ownedParameter_in_Operation(cityModel_PublicBuilding_addEmployee_Person_Person);

	//removeEmployee
	cityModel_PublicBuilding_removeEmployee_Person_Person = factory->createOperation_as_ownedOperation_in_Class(cityModel_PublicBuilding);
	cityModel_PublicBuilding_removeEmployee_Person_Person_person = factory->createParameter_as_ownedParameter_in_Operation(cityModel_PublicBuilding_removeEmployee_Person_Person);
    // PublicBuilding opaque behaviors
	//addEmployee
	cityModel_PublicBuilding_addEmployee = factory->createFunctionBehavior_as_ownedBehavior_in_BehavioredClassifier(cityModel_PublicBuilding);
	

	//removeEmployee
	cityModel_PublicBuilding_removeEmployee = factory->createFunctionBehavior_as_ownedBehavior_in_BehavioredClassifier(cityModel_PublicBuilding);
	

    //-------------------------------------------
	//ResidentialBuilding
	cityModel_ResidentialBuilding = factory->createClass_as_ownedType_in_Package(cityModel, CityModel::CityModelPackage::RESIDENTIALBUILDING_CLASS);
	
    // ResidentialBuilding attributes
	//inhabitants
	cityModel_ResidentialBuilding_inhabitants = factory->createProperty_as_ownedAttribute_in_Class(cityModel_ResidentialBuilding);
    // ResidentialBuilding operations
	//addResident
	cityModel_ResidentialBuilding_addResident_Person_Person = factory->createOperation_as_ownedOperation_in_Class(cityModel_ResidentialBuilding);
	cityModel_ResidentialBuilding_addResident_Person_Person_person = factory->createParameter_as_ownedParameter_in_Operation(cityModel_ResidentialBuilding_addResident_Person_Person);

	//removeResident
	cityModel_ResidentialBuilding_removeResident_Person_Person = factory->createOperation_as_ownedOperation_in_Class(cityModel_ResidentialBuilding);
	cityModel_ResidentialBuilding_removeResident_Person_Person_person = factory->createParameter_as_ownedParameter_in_Operation(cityModel_ResidentialBuilding_removeResident_Person_Person);
    // ResidentialBuilding opaque behaviors
	//addResident
	cityModel_ResidentialBuilding_addResident = factory->createFunctionBehavior_as_ownedBehavior_in_BehavioredClassifier(cityModel_ResidentialBuilding);
	

	//removeResident
	cityModel_ResidentialBuilding_removeResident = factory->createFunctionBehavior_as_ownedBehavior_in_BehavioredClassifier(cityModel_ResidentialBuilding);
	

//-------------------------------------------
//Opaque Behaviors

	//mainProgram
	cityModel_mainProgram = factory->createFunctionBehavior_as_ownedType_in_Package(cityModel);
	
} 

void CityModelPackageImpl::createPackageAssociations(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
	//
	a_CityModel_Person_employees_cityModel_PublicBuilding = factory->createAssociation_as_ownedType_in_Package(cityModel);

    //-------------------------------------------
	//
	a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding = factory->createAssociation_as_ownedType_in_Package(cityModel);

    //-------------------------------------------
	//
	a_cityModel_City_CityModel_Person_persons = factory->createAssociation_as_ownedType_in_Package(cityModel);

    //-------------------------------------------
	//
	a_cityModel_City_CityModel_PublicBuilding_publicBuildings = factory->createAssociation_as_ownedType_in_Package(cityModel);

    //-------------------------------------------
	//
	a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings = factory->createAssociation_as_ownedType_in_Package(cityModel);
}

void CityModelPackageImpl::createPackageDependencies(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
}

void CityModelPackageImpl::createPackageEnumerationLiterals(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
}

void CityModelPackageImpl::createPackageInstanceSpecifications(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
}

void CityModelPackageImpl::createPackageInterfaceRealizations(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
}

void CityModelPackageImpl::createPackageInterfaces(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
}

void CityModelPackageImpl::createPackagePrimitiveTypes(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
}

void CityModelPackageImpl::createPackageStereotypes(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
}

void CityModelPackageImpl::createPackageValueSpecifications(std::shared_ptr<uml::Package> cityModel, std::shared_ptr<uml::umlFactory> factory)
{
	a_CityModel_Person_employees_cityModel_PublicBuilding_Property2_lowerValue_LiteralInteger_A_CityModel_Person_employees_cityModel_PublicBuilding_Property2 = factory->createLiteralInteger_as_ownedElement_in_Element(a_CityModel_Person_employees_cityModel_PublicBuilding_Property2);
	
	a_CityModel_Person_employees_cityModel_PublicBuilding_Property2_upperValue_LiteralUnlimitedNatural_A_CityModel_Person_employees_cityModel_PublicBuilding_Property2 = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(a_CityModel_Person_employees_cityModel_PublicBuilding_Property2);
	
	a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1_lowerValue_LiteralInteger_A_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1 = factory->createLiteralInteger_as_ownedElement_in_Element(a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1);
	
	a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1_upperValue_LiteralUnlimitedNatural_A_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1 = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1);
	
	a_cityModel_City_CityModel_Person_persons_Property3_lowerValue_LiteralInteger_A_cityModel_City_CityModel_Person_persons_Property3 = factory->createLiteralInteger_as_ownedElement_in_Element(a_cityModel_City_CityModel_Person_persons_Property3);
	
	a_cityModel_City_CityModel_Person_persons_Property3_upperValue_LiteralUnlimitedNatural_A_cityModel_City_CityModel_Person_persons_Property3 = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(a_cityModel_City_CityModel_Person_persons_Property3);
	
	a_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1_lowerValue_LiteralInteger_A_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1 = factory->createLiteralInteger_as_ownedElement_in_Element(a_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1);
	
	a_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1_upperValue_LiteralUnlimitedNatural_A_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1 = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(a_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1);
	
	a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3_lowerValue_LiteralInteger_A_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3 = factory->createLiteralInteger_as_ownedElement_in_Element(a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3);
	
	a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3_upperValue_LiteralUnlimitedNatural_A_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3 = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3);
	
	cityModel_Building_address_lowerValue_LiteralInteger_CityModel_Building_address = factory->createLiteralInteger_as_ownedElement_in_Element(cityModel_Building_address);
	
	cityModel_Building_address_upperValue_LiteralUnlimitedNatural_CityModel_Building_address = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(cityModel_Building_address);
	
	cityModel_City_cityName_lowerValue_LiteralInteger_CityModel_City_cityName = factory->createLiteralInteger_as_ownedElement_in_Element(cityModel_City_cityName);
	
	cityModel_City_cityName_upperValue_LiteralUnlimitedNatural_CityModel_City_cityName = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(cityModel_City_cityName);
	
	cityModel_City_persons_lowerValue_LiteralInteger_CityModel_City_persons = factory->createLiteralInteger_as_ownedElement_in_Element(cityModel_City_persons);
	
	cityModel_City_persons_upperValue_LiteralUnlimitedNatural_CityModel_City_persons = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(cityModel_City_persons);
	
	cityModel_City_publicBuildings_lowerValue_LiteralInteger_CityModel_City_publicBuildings = factory->createLiteralInteger_as_ownedElement_in_Element(cityModel_City_publicBuildings);
	
	cityModel_City_publicBuildings_upperValue_LiteralUnlimitedNatural_CityModel_City_publicBuildings = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(cityModel_City_publicBuildings);
	
	cityModel_City_residentialBuildings_lowerValue_LiteralInteger_CityModel_City_residentialBuildings = factory->createLiteralInteger_as_ownedElement_in_Element(cityModel_City_residentialBuildings);
	
	cityModel_City_residentialBuildings_upperValue_LiteralUnlimitedNatural_CityModel_City_residentialBuildings = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(cityModel_City_residentialBuildings);
	
	cityModel_Person_name_lowerValue_LiteralInteger_CityModel_Person_name = factory->createLiteralInteger_as_ownedElement_in_Element(cityModel_Person_name);
	
	cityModel_Person_name_upperValue_LiteralUnlimitedNatural_CityModel_Person_name = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(cityModel_Person_name);
	
	cityModel_PublicBuilding_employees_lowerValue_LiteralInteger_CityModel_PublicBuilding_employees = factory->createLiteralInteger_as_ownedElement_in_Element(cityModel_PublicBuilding_employees);
	
	cityModel_PublicBuilding_employees_upperValue_LiteralUnlimitedNatural_CityModel_PublicBuilding_employees = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(cityModel_PublicBuilding_employees);
	
	cityModel_PublicBuilding_purpose_lowerValue_LiteralInteger_CityModel_PublicBuilding_purpose = factory->createLiteralInteger_as_ownedElement_in_Element(cityModel_PublicBuilding_purpose);
	
	cityModel_PublicBuilding_purpose_upperValue_LiteralUnlimitedNatural_CityModel_PublicBuilding_purpose = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(cityModel_PublicBuilding_purpose);
	
	cityModel_ResidentialBuilding_inhabitants_lowerValue_LiteralInteger_CityModel_ResidentialBuilding_inhabitants = factory->createLiteralInteger_as_ownedElement_in_Element(cityModel_ResidentialBuilding_inhabitants);
	
	cityModel_ResidentialBuilding_inhabitants_upperValue_LiteralUnlimitedNatural_CityModel_ResidentialBuilding_inhabitants = factory->createLiteralUnlimitedNatural_as_ownedElement_in_Element(cityModel_ResidentialBuilding_inhabitants);
	
}

void CityModelPackageImpl::initializePackageContents(std::shared_ptr<uml::Package> cityModel)
{
	if (isInitialized)
	{
		return;
	}
	isInitialized = true;

	// Initialize package
	this->setName(eNAME);
	this->setURI(eNS_URI);

	initializePackageActivities();
	initializePackageClasses();
	initializePackageAssociations();
	initializePackageDependencies();
	initializePackageInstanceSpecifications();
	initializePackageInterfaceRealizations();
	initializePackageInterfaces();
	initializePackageNestedPackages();
	initializePackageStereotypes();
	initializePackageValueSpecifications();
	
}

//ActivityNodes and Edges
void CityModelPackageImpl::initializePackageActivities()
{
}


void CityModelPackageImpl::initializePackageClasses()
{
	std::shared_ptr<uml::Generalization> gen = nullptr;

	//	class Building
	cityModel_Building->setName("Building");
	cityModel_Building->setIsAbstract(true);
	
    // Building attributes
	//address
	cityModel_Building_address->setName("address");
	cityModel_Building_address->setType(PrimitiveTypes::PrimitiveTypesPackage::eInstance()->get_PrimitiveTypes_String());
	cityModel_Building_address->setLower(1);
	cityModel_Building_address->setUpper(1);
	cityModel_Building_address->setIsUnique(true);
	cityModel_Building_address->setAggregation(uml::AggregationKind::NONE);
	cityModel_Building_address->setVisibility(uml::VisibilityKind::PUBLIC);
	
	
	


    //-------------------------------------------
	//	class City
	cityModel_City->setName("City");
	cityModel_City->setIsAbstract(false);
	
    // City attributes
	//cityName
	cityModel_City_cityName->setName("cityName");
	cityModel_City_cityName->setType(PrimitiveTypes::PrimitiveTypesPackage::eInstance()->get_PrimitiveTypes_String());
	cityModel_City_cityName->setLower(1);
	cityModel_City_cityName->setUpper(1);
	cityModel_City_cityName->setIsUnique(true);
	cityModel_City_cityName->setAggregation(uml::AggregationKind::NONE);
	cityModel_City_cityName->setVisibility(uml::VisibilityKind::PUBLIC);
	
	
	
	//persons
	cityModel_City_persons->setName("persons");
	cityModel_City_persons->setType(get_CityModel_Person());
	cityModel_City_persons->setLower(0);
	cityModel_City_persons->setUpper(-1);
	cityModel_City_persons->setIsUnique(true);
	cityModel_City_persons->setAggregation(uml::AggregationKind::COMPOSITE);
	cityModel_City_persons->setVisibility(uml::VisibilityKind::PUBLIC);
	cityModel_City_persons->setAssociation(get_A_cityModel_City_CityModel_Person_persons());
	
	
	//publicBuildings
	cityModel_City_publicBuildings->setName("publicBuildings");
	cityModel_City_publicBuildings->setType(get_CityModel_PublicBuilding());
	cityModel_City_publicBuildings->setLower(0);
	cityModel_City_publicBuildings->setUpper(-1);
	cityModel_City_publicBuildings->setIsUnique(true);
	cityModel_City_publicBuildings->setAggregation(uml::AggregationKind::COMPOSITE);
	cityModel_City_publicBuildings->setVisibility(uml::VisibilityKind::PUBLIC);
	cityModel_City_publicBuildings->setAssociation(get_A_cityModel_City_CityModel_PublicBuilding_publicBuildings());
	
	
	//residentialBuildings
	cityModel_City_residentialBuildings->setName("residentialBuildings");
	cityModel_City_residentialBuildings->setType(get_CityModel_ResidentialBuilding());
	cityModel_City_residentialBuildings->setLower(0);
	cityModel_City_residentialBuildings->setUpper(-1);
	cityModel_City_residentialBuildings->setIsUnique(true);
	cityModel_City_residentialBuildings->setAggregation(uml::AggregationKind::COMPOSITE);
	cityModel_City_residentialBuildings->setVisibility(uml::VisibilityKind::PUBLIC);
	cityModel_City_residentialBuildings->setAssociation(get_A_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings());
	
	

    // City operations
	//changeResidence
	cityModel_City_changeResidence_Person_String_Person_String->setName("changeResidence");
	cityModel_City_changeResidence_Person_String_Person_String->getMethod()->push_back(get_CityModel_City_changeResidence());
	// parameter person
	cityModel_City_changeResidence_Person_String_Person_String_person->setName("person");
	cityModel_City_changeResidence_Person_String_Person_String_person->setType(get_CityModel_Person());
	cityModel_City_changeResidence_Person_String_Person_String_person->setLower(1);
	cityModel_City_changeResidence_Person_String_Person_String_person->setUpper(1);
	cityModel_City_changeResidence_Person_String_Person_String_person->setDirection(uml::ParameterDirectionKind::IN);
	
	
	// parameter residentialBuildingAddress
	
	cityModel_City_changeResidence_Person_String_Person_String_residentialBuildingAddress->setName("residentialBuildingAddress");
	
	cityModel_City_changeResidence_Person_String_Person_String_residentialBuildingAddress->setType(PrimitiveTypes::PrimitiveTypesPackage::eInstance()->get_PrimitiveTypes_String());
	
	cityModel_City_changeResidence_Person_String_Person_String_residentialBuildingAddress->setLower(1);
	
	cityModel_City_changeResidence_Person_String_Person_String_residentialBuildingAddress->setUpper(1);
	
	cityModel_City_changeResidence_Person_String_Person_String_residentialBuildingAddress->setDirection(uml::ParameterDirectionKind::IN);
	
	
	

	//changeWorkplace
	cityModel_City_changeWorkplace_Person_String_Person_String->setName("changeWorkplace");
	cityModel_City_changeWorkplace_Person_String_Person_String->getMethod()->push_back(get_CityModel_City_changeWorkplace());
	// parameter person
	cityModel_City_changeWorkplace_Person_String_Person_String_person->setName("person");
	cityModel_City_changeWorkplace_Person_String_Person_String_person->setType(get_CityModel_Person());
	cityModel_City_changeWorkplace_Person_String_Person_String_person->setLower(1);
	cityModel_City_changeWorkplace_Person_String_Person_String_person->setUpper(1);
	cityModel_City_changeWorkplace_Person_String_Person_String_person->setDirection(uml::ParameterDirectionKind::IN);
	
	
	// parameter publicBuildingAddress
	
	cityModel_City_changeWorkplace_Person_String_Person_String_publicBuildingAddress->setName("publicBuildingAddress");
	
	cityModel_City_changeWorkplace_Person_String_Person_String_publicBuildingAddress->setType(PrimitiveTypes::PrimitiveTypesPackage::eInstance()->get_PrimitiveTypes_String());
	
	cityModel_City_changeWorkplace_Person_String_Person_String_publicBuildingAddress->setLower(1);
	
	cityModel_City_changeWorkplace_Person_String_Person_String_publicBuildingAddress->setUpper(1);
	
	cityModel_City_changeWorkplace_Person_String_Person_String_publicBuildingAddress->setDirection(uml::ParameterDirectionKind::IN);
	
	
	

	//printCityContents
	cityModel_City_printCityContents__->setName("printCityContents");
	cityModel_City_printCityContents__->getMethod()->push_back(get_CityModel_City_printCityContents());
	
    // City opaque behaviors
	//changeResidence
	cityModel_City_changeResidence->setName("changeResidence");
	cityModel_City_changeResidence->setContext(cityModel_City);
	

	//changeWorkplace
	cityModel_City_changeWorkplace->setName("changeWorkplace");
	cityModel_City_changeWorkplace->setContext(cityModel_City);
	

	//printCityContents
	cityModel_City_printCityContents->setName("printCityContents");
	cityModel_City_printCityContents->setContext(cityModel_City);
	

    //-------------------------------------------
	//	class Person
	cityModel_Person->setName("Person");
	cityModel_Person->setIsAbstract(false);
	
    // Person attributes
	//name
	cityModel_Person_name->setName("name");
	cityModel_Person_name->setType(PrimitiveTypes::PrimitiveTypesPackage::eInstance()->get_PrimitiveTypes_String());
	cityModel_Person_name->setLower(1);
	cityModel_Person_name->setUpper(1);
	cityModel_Person_name->setIsUnique(true);
	cityModel_Person_name->setAggregation(uml::AggregationKind::NONE);
	cityModel_Person_name->setVisibility(uml::VisibilityKind::PUBLIC);
	
	
	


    //-------------------------------------------
	//	class PublicBuilding
	cityModel_PublicBuilding->setName("PublicBuilding");
	cityModel_PublicBuilding->setIsAbstract(false);
	
	gen = uml::umlFactory::eInstance()->createGeneralization_as_generalization_in_Classifier(get_CityModel_PublicBuilding());
	gen->setGeneral(get_CityModel_Building());
	
    // PublicBuilding attributes
	//employees
	cityModel_PublicBuilding_employees->setName("employees");
	cityModel_PublicBuilding_employees->setType(get_CityModel_Person());
	cityModel_PublicBuilding_employees->setLower(0);
	cityModel_PublicBuilding_employees->setUpper(-1);
	cityModel_PublicBuilding_employees->setIsUnique(true);
	cityModel_PublicBuilding_employees->setAggregation(uml::AggregationKind::NONE);
	cityModel_PublicBuilding_employees->setVisibility(uml::VisibilityKind::PUBLIC);
	cityModel_PublicBuilding_employees->setAssociation(get_A_CityModel_Person_employees_cityModel_PublicBuilding());
	
	
	//purpose
	cityModel_PublicBuilding_purpose->setName("purpose");
	cityModel_PublicBuilding_purpose->setType(PrimitiveTypes::PrimitiveTypesPackage::eInstance()->get_PrimitiveTypes_String());
	cityModel_PublicBuilding_purpose->setLower(1);
	cityModel_PublicBuilding_purpose->setUpper(1);
	cityModel_PublicBuilding_purpose->setIsUnique(true);
	cityModel_PublicBuilding_purpose->setAggregation(uml::AggregationKind::NONE);
	cityModel_PublicBuilding_purpose->setVisibility(uml::VisibilityKind::PUBLIC);
	
	
	

    // PublicBuilding operations
	//addEmployee
	cityModel_PublicBuilding_addEmployee_Person_Person->setName("addEmployee");
	cityModel_PublicBuilding_addEmployee_Person_Person->getMethod()->push_back(get_CityModel_PublicBuilding_addEmployee());
	// parameter person
	cityModel_PublicBuilding_addEmployee_Person_Person_person->setName("person");
	cityModel_PublicBuilding_addEmployee_Person_Person_person->setType(get_CityModel_Person());
	cityModel_PublicBuilding_addEmployee_Person_Person_person->setLower(1);
	cityModel_PublicBuilding_addEmployee_Person_Person_person->setUpper(1);
	cityModel_PublicBuilding_addEmployee_Person_Person_person->setDirection(uml::ParameterDirectionKind::IN);
	
	

	//removeEmployee
	cityModel_PublicBuilding_removeEmployee_Person_Person->setName("removeEmployee");
	cityModel_PublicBuilding_removeEmployee_Person_Person->getMethod()->push_back(get_CityModel_PublicBuilding_removeEmployee());
	// parameter person
	cityModel_PublicBuilding_removeEmployee_Person_Person_person->setName("person");
	cityModel_PublicBuilding_removeEmployee_Person_Person_person->setType(get_CityModel_Person());
	cityModel_PublicBuilding_removeEmployee_Person_Person_person->setLower(1);
	cityModel_PublicBuilding_removeEmployee_Person_Person_person->setUpper(1);
	cityModel_PublicBuilding_removeEmployee_Person_Person_person->setDirection(uml::ParameterDirectionKind::IN);
	
	
    // PublicBuilding opaque behaviors
	//addEmployee
	cityModel_PublicBuilding_addEmployee->setName("addEmployee");
	cityModel_PublicBuilding_addEmployee->setContext(cityModel_PublicBuilding);
	

	//removeEmployee
	cityModel_PublicBuilding_removeEmployee->setName("removeEmployee");
	cityModel_PublicBuilding_removeEmployee->setContext(cityModel_PublicBuilding);
	

    //-------------------------------------------
	//	class ResidentialBuilding
	cityModel_ResidentialBuilding->setName("ResidentialBuilding");
	cityModel_ResidentialBuilding->setIsAbstract(false);
	
	gen = uml::umlFactory::eInstance()->createGeneralization_as_generalization_in_Classifier(get_CityModel_ResidentialBuilding());
	gen->setGeneral(get_CityModel_Building());
	
    // ResidentialBuilding attributes
	//inhabitants
	cityModel_ResidentialBuilding_inhabitants->setName("inhabitants");
	cityModel_ResidentialBuilding_inhabitants->setType(get_CityModel_Person());
	cityModel_ResidentialBuilding_inhabitants->setLower(0);
	cityModel_ResidentialBuilding_inhabitants->setUpper(-1);
	cityModel_ResidentialBuilding_inhabitants->setIsUnique(true);
	cityModel_ResidentialBuilding_inhabitants->setAggregation(uml::AggregationKind::NONE);
	cityModel_ResidentialBuilding_inhabitants->setVisibility(uml::VisibilityKind::PUBLIC);
	cityModel_ResidentialBuilding_inhabitants->setAssociation(get_A_CityModel_Person_inhabitants_cityModel_ResidentialBuilding());
	
	

    // ResidentialBuilding operations
	//addResident
	cityModel_ResidentialBuilding_addResident_Person_Person->setName("addResident");
	cityModel_ResidentialBuilding_addResident_Person_Person->getMethod()->push_back(get_CityModel_ResidentialBuilding_addResident());
	// parameter person
	cityModel_ResidentialBuilding_addResident_Person_Person_person->setName("person");
	cityModel_ResidentialBuilding_addResident_Person_Person_person->setType(get_CityModel_Person());
	cityModel_ResidentialBuilding_addResident_Person_Person_person->setLower(1);
	cityModel_ResidentialBuilding_addResident_Person_Person_person->setUpper(1);
	cityModel_ResidentialBuilding_addResident_Person_Person_person->setDirection(uml::ParameterDirectionKind::IN);
	
	

	//removeResident
	cityModel_ResidentialBuilding_removeResident_Person_Person->setName("removeResident");
	cityModel_ResidentialBuilding_removeResident_Person_Person->getMethod()->push_back(get_CityModel_ResidentialBuilding_removeResident());
	// parameter person
	cityModel_ResidentialBuilding_removeResident_Person_Person_person->setName("person");
	cityModel_ResidentialBuilding_removeResident_Person_Person_person->setType(get_CityModel_Person());
	cityModel_ResidentialBuilding_removeResident_Person_Person_person->setLower(1);
	cityModel_ResidentialBuilding_removeResident_Person_Person_person->setUpper(1);
	cityModel_ResidentialBuilding_removeResident_Person_Person_person->setDirection(uml::ParameterDirectionKind::IN);
	
	
    // ResidentialBuilding opaque behaviors
	//addResident
	cityModel_ResidentialBuilding_addResident->setName("addResident");
	cityModel_ResidentialBuilding_addResident->setContext(cityModel_ResidentialBuilding);
	

	//removeResident
	cityModel_ResidentialBuilding_removeResident->setName("removeResident");
	cityModel_ResidentialBuilding_removeResident->setContext(cityModel_ResidentialBuilding);
	

//-------------------------------------------
//Opaque Behaviors

	//mainProgram
	cityModel_mainProgram->setName("mainProgram");
	
	{
		//apply Stereotype MainBehavior
		std::shared_ptr<UML4CPPProfile::MainBehavior> stereo(UML4CPPProfile::UML4CPPProfileFactory::eInstance()->createMainBehavior());
		stereo->setBase_Behavior(cityModel_mainProgram);
		cityModel_mainProgram->applyStereotype(stereo);
	}
	
}

void CityModelPackageImpl::initializePackageAssociations()
{
	//
	a_CityModel_Person_employees_cityModel_PublicBuilding->setName("");
	a_CityModel_Person_employees_cityModel_PublicBuilding->getMemberEnd()->add(get_CityModel_PublicBuilding_employees());
	a_CityModel_Person_employees_cityModel_PublicBuilding->getMemberEnd()->add(get_A_CityModel_Person_employees_cityModel_PublicBuilding_Property2());
	

    //-------------------------------------------
	//
	a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding->setName("");
	a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding->getMemberEnd()->add(get_CityModel_ResidentialBuilding_inhabitants());
	a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding->getMemberEnd()->add(get_A_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1());
	

    //-------------------------------------------
	//
	a_cityModel_City_CityModel_Person_persons->setName("");
	a_cityModel_City_CityModel_Person_persons->getMemberEnd()->add(get_A_cityModel_City_CityModel_Person_persons_Property3());
	a_cityModel_City_CityModel_Person_persons->getMemberEnd()->add(get_CityModel_City_persons());
	

    //-------------------------------------------
	//
	a_cityModel_City_CityModel_PublicBuilding_publicBuildings->setName("");
	a_cityModel_City_CityModel_PublicBuilding_publicBuildings->getMemberEnd()->add(get_A_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1());
	a_cityModel_City_CityModel_PublicBuilding_publicBuildings->getMemberEnd()->add(get_CityModel_City_publicBuildings());
	

    //-------------------------------------------
	//
	a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings->setName("");
	a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings->getMemberEnd()->add(get_A_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3());
	a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings->getMemberEnd()->add(get_CityModel_City_residentialBuildings());
	
}

void CityModelPackageImpl::initializePackageDependencies()
{
}

void CityModelPackageImpl::initializePackageInstanceSpecifications()
{
}


void CityModelPackageImpl::initializePackageInterfaceRealizations()
{
}

void CityModelPackageImpl::initializePackageInterfaces()
{
}

void CityModelPackageImpl::initializePackageNestedPackages()
{
}

void CityModelPackageImpl::initializePackageStereotypes()
{
}

void CityModelPackageImpl::initializePackageValueSpecifications()
{
	a_CityModel_Person_employees_cityModel_PublicBuilding_Property2_lowerValue_LiteralInteger_A_CityModel_Person_employees_cityModel_PublicBuilding_Property2->setValue(0);

	a_CityModel_Person_employees_cityModel_PublicBuilding_Property2_upperValue_LiteralUnlimitedNatural_A_CityModel_Person_employees_cityModel_PublicBuilding_Property2->setValue(1);

	a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1_lowerValue_LiteralInteger_A_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1->setValue(0);

	a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1_upperValue_LiteralUnlimitedNatural_A_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1->setValue(1);

	a_cityModel_City_CityModel_Person_persons_Property3_lowerValue_LiteralInteger_A_cityModel_City_CityModel_Person_persons_Property3->setValue(1);

	a_cityModel_City_CityModel_Person_persons_Property3_upperValue_LiteralUnlimitedNatural_A_cityModel_City_CityModel_Person_persons_Property3->setValue(1);

	a_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1_lowerValue_LiteralInteger_A_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1->setValue(1);

	a_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1_upperValue_LiteralUnlimitedNatural_A_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1->setValue(1);

	a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3_lowerValue_LiteralInteger_A_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3->setValue(1);

	a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3_upperValue_LiteralUnlimitedNatural_A_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3->setValue(1);

	cityModel_Building_address_lowerValue_LiteralInteger_CityModel_Building_address->setValue(1);

	cityModel_Building_address_upperValue_LiteralUnlimitedNatural_CityModel_Building_address->setValue(1);

	cityModel_City_cityName_lowerValue_LiteralInteger_CityModel_City_cityName->setValue(1);

	cityModel_City_cityName_upperValue_LiteralUnlimitedNatural_CityModel_City_cityName->setValue(1);

	cityModel_City_persons_lowerValue_LiteralInteger_CityModel_City_persons->setValue(0);

	cityModel_City_persons_upperValue_LiteralUnlimitedNatural_CityModel_City_persons->setValue(-1);

	cityModel_City_publicBuildings_lowerValue_LiteralInteger_CityModel_City_publicBuildings->setValue(0);

	cityModel_City_publicBuildings_upperValue_LiteralUnlimitedNatural_CityModel_City_publicBuildings->setValue(-1);

	cityModel_City_residentialBuildings_lowerValue_LiteralInteger_CityModel_City_residentialBuildings->setValue(0);

	cityModel_City_residentialBuildings_upperValue_LiteralUnlimitedNatural_CityModel_City_residentialBuildings->setValue(-1);

	cityModel_Person_name_lowerValue_LiteralInteger_CityModel_Person_name->setValue(1);

	cityModel_Person_name_upperValue_LiteralUnlimitedNatural_CityModel_Person_name->setValue(1);

	cityModel_PublicBuilding_employees_lowerValue_LiteralInteger_CityModel_PublicBuilding_employees->setValue(0);

	cityModel_PublicBuilding_employees_upperValue_LiteralUnlimitedNatural_CityModel_PublicBuilding_employees->setValue(-1);

	cityModel_PublicBuilding_purpose_lowerValue_LiteralInteger_CityModel_PublicBuilding_purpose->setValue(1);

	cityModel_PublicBuilding_purpose_upperValue_LiteralUnlimitedNatural_CityModel_PublicBuilding_purpose->setValue(1);

	cityModel_ResidentialBuilding_inhabitants_lowerValue_LiteralInteger_CityModel_ResidentialBuilding_inhabitants->setValue(0);

	cityModel_ResidentialBuilding_inhabitants_upperValue_LiteralUnlimitedNatural_CityModel_ResidentialBuilding_inhabitants->setValue(-1);
}

std::shared_ptr<uml::Association> CityModelPackageImpl::get_A_CityModel_Person_employees_cityModel_PublicBuilding()
{
	return a_CityModel_Person_employees_cityModel_PublicBuilding;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_A_CityModel_Person_employees_cityModel_PublicBuilding_Property2()
{
	return a_CityModel_Person_employees_cityModel_PublicBuilding_Property2;
}
std::shared_ptr<uml::Association> CityModelPackageImpl::get_A_CityModel_Person_inhabitants_cityModel_ResidentialBuilding()
{
	return a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_A_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1()
{
	return a_CityModel_Person_inhabitants_cityModel_ResidentialBuilding_Property1;
}
std::shared_ptr<uml::Association> CityModelPackageImpl::get_A_cityModel_City_CityModel_Person_persons()
{
	return a_cityModel_City_CityModel_Person_persons;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_A_cityModel_City_CityModel_Person_persons_Property3()
{
	return a_cityModel_City_CityModel_Person_persons_Property3;
}
std::shared_ptr<uml::Association> CityModelPackageImpl::get_A_cityModel_City_CityModel_PublicBuilding_publicBuildings()
{
	return a_cityModel_City_CityModel_PublicBuilding_publicBuildings;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_A_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1()
{
	return a_cityModel_City_CityModel_PublicBuilding_publicBuildings_Property1;
}
std::shared_ptr<uml::Association> CityModelPackageImpl::get_A_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings()
{
	return a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_A_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3()
{
	return a_cityModel_City_CityModel_ResidentialBuilding_residentialBuildings_Property3;
}
std::shared_ptr<uml::Class> CityModelPackageImpl::get_CityModel_Building()
{
	return cityModel_Building;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_CityModel_Building_address()
{
	return cityModel_Building_address;
}
std::shared_ptr<uml::Class> CityModelPackageImpl::get_CityModel_City()
{
	return cityModel_City;
}
std::shared_ptr<uml::FunctionBehavior> CityModelPackageImpl::get_CityModel_City_changeResidence()
{
	return cityModel_City_changeResidence;
}
std::shared_ptr<uml::Operation> CityModelPackageImpl::get_CityModel_City_changeResidence_Person_String_Person_String()
{
	return cityModel_City_changeResidence_Person_String_Person_String;
}
std::shared_ptr<uml::Parameter> CityModelPackageImpl::get_CityModel_City_changeResidence_Person_String_Person_String_person()
{
	return cityModel_City_changeResidence_Person_String_Person_String_person;
}
std::shared_ptr<uml::Parameter> CityModelPackageImpl::get_CityModel_City_changeResidence_Person_String_Person_String_residentialBuildingAddress()
{
	return cityModel_City_changeResidence_Person_String_Person_String_residentialBuildingAddress;
}
std::shared_ptr<uml::FunctionBehavior> CityModelPackageImpl::get_CityModel_City_changeWorkplace()
{
	return cityModel_City_changeWorkplace;
}
std::shared_ptr<uml::Operation> CityModelPackageImpl::get_CityModel_City_changeWorkplace_Person_String_Person_String()
{
	return cityModel_City_changeWorkplace_Person_String_Person_String;
}
std::shared_ptr<uml::Parameter> CityModelPackageImpl::get_CityModel_City_changeWorkplace_Person_String_Person_String_person()
{
	return cityModel_City_changeWorkplace_Person_String_Person_String_person;
}
std::shared_ptr<uml::Parameter> CityModelPackageImpl::get_CityModel_City_changeWorkplace_Person_String_Person_String_publicBuildingAddress()
{
	return cityModel_City_changeWorkplace_Person_String_Person_String_publicBuildingAddress;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_CityModel_City_cityName()
{
	return cityModel_City_cityName;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_CityModel_City_persons()
{
	return cityModel_City_persons;
}
std::shared_ptr<uml::FunctionBehavior> CityModelPackageImpl::get_CityModel_City_printCityContents()
{
	return cityModel_City_printCityContents;
}
std::shared_ptr<uml::Operation> CityModelPackageImpl::get_CityModel_City_printCityContents__()
{
	return cityModel_City_printCityContents__;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_CityModel_City_publicBuildings()
{
	return cityModel_City_publicBuildings;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_CityModel_City_residentialBuildings()
{
	return cityModel_City_residentialBuildings;
}
std::shared_ptr<uml::Class> CityModelPackageImpl::get_CityModel_Person()
{
	return cityModel_Person;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_CityModel_Person_name()
{
	return cityModel_Person_name;
}
std::shared_ptr<uml::Class> CityModelPackageImpl::get_CityModel_PublicBuilding()
{
	return cityModel_PublicBuilding;
}
std::shared_ptr<uml::FunctionBehavior> CityModelPackageImpl::get_CityModel_PublicBuilding_addEmployee()
{
	return cityModel_PublicBuilding_addEmployee;
}
std::shared_ptr<uml::Operation> CityModelPackageImpl::get_CityModel_PublicBuilding_addEmployee_Person_Person()
{
	return cityModel_PublicBuilding_addEmployee_Person_Person;
}
std::shared_ptr<uml::Parameter> CityModelPackageImpl::get_CityModel_PublicBuilding_addEmployee_Person_Person_person()
{
	return cityModel_PublicBuilding_addEmployee_Person_Person_person;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_CityModel_PublicBuilding_employees()
{
	return cityModel_PublicBuilding_employees;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_CityModel_PublicBuilding_purpose()
{
	return cityModel_PublicBuilding_purpose;
}
std::shared_ptr<uml::FunctionBehavior> CityModelPackageImpl::get_CityModel_PublicBuilding_removeEmployee()
{
	return cityModel_PublicBuilding_removeEmployee;
}
std::shared_ptr<uml::Operation> CityModelPackageImpl::get_CityModel_PublicBuilding_removeEmployee_Person_Person()
{
	return cityModel_PublicBuilding_removeEmployee_Person_Person;
}
std::shared_ptr<uml::Parameter> CityModelPackageImpl::get_CityModel_PublicBuilding_removeEmployee_Person_Person_person()
{
	return cityModel_PublicBuilding_removeEmployee_Person_Person_person;
}
std::shared_ptr<uml::Class> CityModelPackageImpl::get_CityModel_ResidentialBuilding()
{
	return cityModel_ResidentialBuilding;
}
std::shared_ptr<uml::FunctionBehavior> CityModelPackageImpl::get_CityModel_ResidentialBuilding_addResident()
{
	return cityModel_ResidentialBuilding_addResident;
}
std::shared_ptr<uml::Operation> CityModelPackageImpl::get_CityModel_ResidentialBuilding_addResident_Person_Person()
{
	return cityModel_ResidentialBuilding_addResident_Person_Person;
}
std::shared_ptr<uml::Parameter> CityModelPackageImpl::get_CityModel_ResidentialBuilding_addResident_Person_Person_person()
{
	return cityModel_ResidentialBuilding_addResident_Person_Person_person;
}
std::shared_ptr<uml::Property> CityModelPackageImpl::get_CityModel_ResidentialBuilding_inhabitants()
{
	return cityModel_ResidentialBuilding_inhabitants;
}
std::shared_ptr<uml::FunctionBehavior> CityModelPackageImpl::get_CityModel_ResidentialBuilding_removeResident()
{
	return cityModel_ResidentialBuilding_removeResident;
}
std::shared_ptr<uml::Operation> CityModelPackageImpl::get_CityModel_ResidentialBuilding_removeResident_Person_Person()
{
	return cityModel_ResidentialBuilding_removeResident_Person_Person;
}
std::shared_ptr<uml::Parameter> CityModelPackageImpl::get_CityModel_ResidentialBuilding_removeResident_Person_Person_person()
{
	return cityModel_ResidentialBuilding_removeResident_Person_Person_person;
}
std::shared_ptr<uml::FunctionBehavior> CityModelPackageImpl::get_CityModel_mainProgram()
{
	return cityModel_mainProgram;
}




