#include "CityModel/impl/PersonImpl.hpp"

#ifdef NDEBUG
  #define DEBUG_MESSAGE(a) /**/
#else
  #define DEBUG_MESSAGE(a) a
#endif

#include <iostream>


#include "abstractDataTypes/Any.hpp"
#include "abstractDataTypes/SubsetUnion.hpp"
#include "util/util.hpp"
#include "uml/Property.hpp"
#include "uml/Operation.hpp"
#include "uml/Parameter.hpp"
#include "CityModel/CityModelFactory.hpp"
#include "CityModel/impl/CityModelPackageImpl.hpp"
#include "uml/Class.hpp"

//Types included from attributes, operation parameters, imports and composite owner classes

//Packges and Factories included from types of attributes, operation parameters, imports and composite owner classes

//Packages of included Enumerations

//Includes of PluginFramework (if required)

//Includes of OpaqueBevaiors (if required)

//Includes from InstanceValues (if required)

//Includes from Ports typed by interfaces (if required)

//Includes from roles of ConnectorEnds (if required)

using namespace CityModel;

//*********************************
// Constructor / Destructor
//*********************************
PersonImpl::PersonImpl()
{
	/*
	NOTE: Due to virtual inheritance, base class constrcutors may not be called correctly
	*/
	DEBUG_MESSAGE(std::cout<<"Person is created..."<<std::endl;)
	//***********************************
	// init Get Set
	//getter init
		//Property name
		m_getterMap.insert(std::pair<long long,std::function<Any()>>(1050021574,[this](){ return eAny(this->getName());}));
	
	
	//setter init
	//Property name
		m_setterMap.insert(std::pair<long long,std::function<void(Any)>>(1050021574,[this](Any object){this->setName(object->get<std::string>());}));
	
	
	
	
}


PersonImpl::~PersonImpl()
{
	DEBUG_MESSAGE(std::cout<<"Person is destroyed..."<<std::endl;)
}

PersonImpl::PersonImpl(const PersonImpl & obj):PersonImpl()
{
	*this = obj;
}

std::shared_ptr<ecore::EObject>  PersonImpl::copy() const
{
	std::shared_ptr<PersonImpl> element(new PersonImpl());
	*element=(*this);
	element->setThisPersonPtr(element);
	return element;
}

PersonImpl& PersonImpl::operator=(const PersonImpl & obj)
{
	//call overloaded =Operator for each base class
	uml::ElementImpl::operator=(obj);
	//create copy of all Attributes
	#ifdef SHOW_COPIES
	std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\r\ncopy Person "<< this << "\r\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ " << std::endl;
	#endif
	instantiate();

	//copy attributes with no containment (soft copy)
	m_name = obj.getName();

	//clone attributes with containment (deep copy)

	return *this;
}


std::shared_ptr<uml::Class> PersonImpl::getMetaClass()
{
	return CityModelPackageImpl::eInstance()->get_CityModel_Person();
}

void PersonImpl::instantiate()
{   
	
}

void PersonImpl::destroy()
{	

	//Erase properties
	
	//Erase back reference to owner
}

//*********************************
// Attribute Setter Getter
//*********************************
void PersonImpl::setName(std::string _name)
{
	m_name = _name;
	
}
std::string PersonImpl::getName() const 
{

	return m_name;
}

//*********************************
// Union Getter
//*********************************



//*********************************
// Operations
//*********************************

//*********************************
// Structural Feature Getter/Setter
//*********************************
//Get
Any PersonImpl::get(std::shared_ptr<uml::Property> _property) const
{
	std::string qualifiedName = _property->getQualifiedName();
    return this->get(qualifiedName);
}

Any PersonImpl::get(std::string _qualifiedName) const
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    return this->get(uID);
}

Any PersonImpl::get(long long _uID) const
{
	std::map<long long, std::function<Any()>>::const_iterator iter = m_getterMap.find(_uID);
    if(iter != m_getterMap.cend())
    {
        //invoke the getter function
        return iter->second();
    }

	return eAny(nullptr);
}

//Set
void PersonImpl::set(std::shared_ptr<uml::Property> _property, Any value)
{
	std::string qualifiedName = _property->getQualifiedName();
    this->set(qualifiedName, value);
}

void PersonImpl::set(std::string _qualifiedName, Any value)
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    this->set(uID, value);
}

void PersonImpl::set(long long _uID, Any value)
{
	std::map<long long, std::function<void(Any)>>::const_iterator iter = m_setterMap.find(_uID);
    if(iter != m_setterMap.cend())
    {
        //invoke the setter function
        iter->second(value);
    }
}

//Unset
void PersonImpl::unset(std::shared_ptr<uml::Property> _property)
{
	std::string qualifiedName = _property->getQualifiedName();
    this->unset(qualifiedName);
}

void PersonImpl::unset(std::string _qualifiedName)
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    this->unset(uID);
}

void PersonImpl::unset(long long _uID)
{
	std::map<long long, std::function<void()>>::const_iterator iter = m_unsetterMap.find(_uID);
    if(iter != m_unsetterMap.cend())
    {
        //invoke the unsetter function
        iter->second();
    }
}


//*********************************
// Operation Invoction
//*********************************
//Invoke
Any PersonImpl::invoke(std::shared_ptr<uml::Operation> _operation, std::shared_ptr<Bag<Any>> _arguments)
{
	std::string qualifiedName = _operation->getQualifiedName();

	for(unsigned int i = 0; i < _operation->getOwnedParameter()->size(); i++)
	{
		qualifiedName += "_" + _operation->getOwnedParameter()->at(i)->getType()->getName();
	}

    return this->invoke(qualifiedName, _arguments);
}

Any PersonImpl::invoke(std::string _qualifiedName, std::shared_ptr<Bag<Any>> _arguments)
{
	long long uID = util::Util::polynomialRollingHash(_qualifiedName);
    return this->invoke(uID, _arguments);
}

Any PersonImpl::invoke(long long _uID, std::shared_ptr<Bag<Any>> _arguments)
{
	std::map<long long, std::function<Any(std::shared_ptr<Bag<Any>>)>>::const_iterator iter = m_invocationMap.find(_uID);
    if(iter != m_invocationMap.cend())
    {
        //invoke the operation
        return iter->second(_arguments);
    }
	
	return eAny(nullptr);
}

std::shared_ptr<Person> PersonImpl::getThisPersonPtr()
{
	return m_thisPersonPtr.lock();
}
void PersonImpl::setThisPersonPtr(std::weak_ptr<Person> thisPersonPtr)
{
	m_thisPersonPtr = thisPersonPtr;
	setThisElementPtr(thisPersonPtr);
}
